@@ -48,12 +48,12 @@ use base qw(LaTeXML::Post::Processor);
 sub new {
   my ($class, %options) = @_;
   my $self = $class->SUPER::new(%options);
-  $$self{dppt} = (($options{dpi} || 90) / 72.0);    # Dots per point.
+  $$self{dppt}            = (($options{dpi} || 90) / 72.0);    # Dots per point.
   $$self{ignore_options}  = $options{ignore_options}  || [];
   $$self{trivial_scaling} = $options{trivial_scaling} || 1;
   $$self{graphics_types}  = $options{graphics_types}
     || [qw(svg png gif jpg jpeg
-      eps ps ai pdf)];
+      eps ps postscript ai pdf)];
   $$self{type_properties} = $options{type_properties}
     || {
     ai => { destination_type => 'png',
@@ -89,8 +89,8 @@ sub process {
   my ($self, $doc, @nodes) = @_;
   local $LaTeXML::Post::Graphics::SEARCHPATHS
     = [map { pathname_canonical($_) } $self->findGraphicsPaths($doc), $doc->getSearchPaths];
-  NoteProgressDetailed(" [Using graphicspaths: "
-      . join(', ', @$LaTeXML::Post::Graphics::SEARCHPATHS) . "]");
+  Debug(" [Using graphicspaths: "
+      . join(', ', @$LaTeXML::Post::Graphics::SEARCHPATHS) . "]") if $LaTeXML::DEBUG{images};
   foreach my $node (@nodes) {
     $self->processGraphic($doc, $node); }
   $doc->closeCache;    # If opened.
@@ -113,7 +113,7 @@ sub findGraphicsPaths {
 
 sub getGraphicsSourceTypes {
   my ($self) = @_;
-  return @{ $$self{graphics_types} }; }
+  return @{ $$self{graphics_types} }, map { uc($_); } @{ $$self{graphics_types} }; }
 
 # Return the pathname to an appropriate image.
 sub findGraphicFile {
@@ -122,7 +122,7 @@ sub findGraphicFile {
     # Find all acceptable image files, in order of search paths
     my ($dir, $name, $reqtype) = pathname_split($source);
     # Ignore the requested type? Or should it increase desirability?
-    my $file = pathname_concat($dir, $name);
+    my $file  = pathname_concat($dir, $name);
     my @paths = pathname_findall($file, paths => $LaTeXML::Post::Graphics::SEARCHPATHS,
       # accept empty type, incase bad type name, but actual file's content is known type.
       types => ['', $self->getGraphicsSourceTypes]);
@@ -130,8 +130,8 @@ sub findGraphicFile {
     # Now, find the first image that is either the correct type,
     # or has the most desirable type mapping
     foreach my $path (@paths) {
-      my $type  = pathname_type($path);
-      my $props = $$self{type_properties}{$type};
+      my $type         = pathname_type($path);
+      my $props        = $$self{type_properties}{$type};
       my $desirability = $$props{desirability} || ($type eq ($$props{destination_type} || 'notype') ? 10 : 0);
       if ($desirability > $best) {
         $best     = $desirability;
@@ -163,8 +163,10 @@ sub getTypeProperties {
 # width and height.
 sub setGraphicSrc {
   my ($self, $node, $src, $width, $height) = @_;
-  $node->setAttribute('imagesrc',    $src);
-  $node->setAttribute('imagewidth',  $width) if defined $width;
+  # If we are on windows, the $src path will be used for a URI context from the 'imagesrc' attribute,
+  # so we can already switch it to the canonical slashified form
+  $node->setAttribute('imagesrc',    pathname_to_url($src));
+  $node->setAttribute('imagewidth',  $width)  if defined $width;
   $node->setAttribute('imageheight', $height) if defined $height;
   return; }
 
@@ -196,9 +198,9 @@ sub transformGraphic {
   return Warn('unexpected', 'graphics_format', undef,
     "Don't know what to do with graphics file format '$source'") unless %properties;
   my $type = $properties{destination_type} || $srctype;
-  my $key = (ref $self) . ':' . join('|', "$reldir$name.$srctype.$type",
+  my $key  = (ref $self) . ':' . join('|', "$reldir$name.$srctype.$type",
     map { join(' ', @$_) } @$transform);
-  NoteProgressDetailed("\n[Processing $source as key=$key]");
+  Debug("Processing $source as key=$key") if $LaTeXML::DEBUG{images};
 
   my $dest = $self->desiredResourcePathname($doc, $node, $source, $type);
   if (my $prev = $doc->cacheLookup($key)) {                 # Image was processed on previous run?
@@ -210,7 +212,8 @@ sub transformGraphic {
       if ((!defined $dest) || ($cached eq $dest)) {
         my $absdest = pathname_absolute($cached, $doc->getDestinationDirectory);
         if (pathname_timestamp($source) <= pathname_timestamp($absdest)) {
-          NoteProgressDetailed(" [Reuse $cached @ " . ($width || '?') . " x " . ($height || '?') . "]");
+          Debug(" [Reuse $cached @ " . ($width || '?') . " x " . ($height || '?') . "]")
+            if $LaTeXML::DEBUG{images};
           return ($cached, $width, $height); } } } }
   # Trivial scaling case: Use original image with (at most) different width & height.
   my $triv_scaling = $$self{trivial_scaling} && ($type eq $srctype)
@@ -249,10 +252,10 @@ sub transformGraphic {
     # Now IFF that is a valid relative path WITHIN the site directory, we'll use it.
     # Otherwise, we'd better fall back to a generated name.
     if (!pathname_is_contained($absdest, $doc->getSiteDirectory)) {
-      $dest = $self->generateResourcePathname($doc, $node, $source, $type);
+      $dest    = $self->generateResourcePathname($doc, $node, $source, $type);
       $absdest = $doc->checkDestination($dest); }
 
-    NoteProgressDetailed(" [Destination $absdest]");
+    Debug(" [Destination $absdest]") if $LaTeXML::DEBUG{images};
     ($width, $height) = image_graphicx_trivial($source, $transform, ddpt => $$self{ddpt});
     if (!($width && $height)) {
       if (!image_can_image()) {
@@ -265,23 +268,24 @@ sub transformGraphic {
           "Couldn't get usable image size for $source"); } }
     pathname_copy($source, $absdest)
       or Warn('I/O', $absdest, undef, "Couldn't copy $source to $absdest", "Response was: $!");
-    NoteProgressDetailed(" [Copied to $dest @ " . ($width || '?') . " x " . ($height || '?') . "]"); }
+    Debug(" [Copied to $dest @ " . ($width || '?') . " x " . ($height || '?') . "]")
+      if $LaTeXML::DEBUG{images}; }
   else {
     # With a complex transformation, we really needs a new name (well, don't we?)
     $dest = $self->generateResourcePathname($doc, $node, $source, $type) unless $dest;
     my $absdest = $doc->checkDestination($dest);
-    NoteProgressDetailed(" [Destination $absdest]");
+    Debug(" [Destination $absdest]") if $LaTeXML::DEBUG{images};
     ($image, $width, $height) = image_graphicx_complex($source, $transform,
       ddpt => $$self{ddpt}, background => $$self{background}, %properties);
     if (!($image && $width && $height)) {
       Warn('expected', 'image', undef,
         "Couldn't get usable image for $source");
       return; }
-    NoteProgressDetailed(" [Writing to $absdest]");
+    Debug(" [Writing to $absdest]") if $LaTeXML::DEBUG{images};
     image_write($image, $absdest) or return; }
 
   $doc->cacheStore($key, "$dest|" . ($width || '') . '|' . ($height || ''));
-  NoteProgressDetailed(" [done with $key]");
+  Debug(" [done with $key]") if $LaTeXML::DEBUG{images};
   return ($dest, $width, $height); }
 
 #======================================================================
