@@ -40,39 +40,43 @@ sub new {
 
 sub toProcess {
   my ($self, $doc) = @_;
-  return $doc->findnode('//ltx:bibliography'); }
+  return $doc->findnodes('//ltx:bibliography'); }
 
+# Whooo, this is turning into a confusing API!
+# Potentially multiple biblist's in each document
+# potentially each one split into multiple output documents (need that option!)
+# Inevitably duplicates?
 sub process {
-  my ($self, $doc, $bib) = @_;
+  my ($self, $doc, @bibs) = @_;
+  my @docs = ($doc);
+  foreach my $bib (@bibs) {
+    next if $doc->findnodes('.//ltx:bibitem', $bib);    # Already populated?
 
-  my @bib = ($doc);
-  return $doc if $doc->findnodes('//ltx:bibitem', $bib);    # Already populated?
+    local $LaTeXML::Post::MakeBibliography::NUMBER = 0;
+    local %LaTeXML::Post::MakeBibliography::STYLE =
+      (map { ($_ => $bib->getAttribute($_)) } qw(bibstyle citestyle sort));
 
-  local $LaTeXML::Post::MakeBibliography::NUMBER = 0;
-  local %LaTeXML::Post::MakeBibliography::STYLE =
-    (map { ($_ => $bib->getAttribute($_)) } qw(bibstyle citestyle sort));
+    my $entries = $self->getBibEntries($doc, $bib);
+    # Remove any bibentry's (these should have been converted to bibitems)
+    $doc->removeNodes($doc->findnodes('//ltx:bibentry'));
+    foreach my $biblist ($doc->findnodes('//ltx:biblist')) {
+      $doc->removeNodes($biblist) unless element_nodes($biblist); }
 
-  my $entries = $self->getBibEntries($doc);
-  # Remove any bibentry's (these should have been converted to bibitems)
-  $doc->removeNodes($doc->findnodes('//ltx:bibentry'));
-  foreach my $biblist ($doc->findnodes('//ltx:biblist')) {
-    $doc->removeNodes($biblist)
-      unless grep { $_->nodeType == XML_ELEMENT_NODE } $biblist->childNodes; }
-
-  if ($$self{split}) {
-    # Separate by initial.
-    my $split = {};
-    foreach my $sortkey (keys %$entries) {
-      my $entry = $$entries{$sortkey};
-      $$split{ $$entry{initial} }{$sortkey} = $entry; }
-    @bib = map { $self->rescan($_) }
-      $self->makeSubCollectionDocuments($doc, $bib,
-      map { ($_ => $self->makeBibliographyList($doc, $_, $$split{$_})) }
-        sort keys %$split); }
-  else {
-    $doc->addNodes($bib, $self->makeBibliographyList($doc, undef, $entries));
-    @bib = ($self->rescan($doc)); }
-  return @bib; }
+    if ($$self{split}) {
+      # Separate by initial.
+      my $split = {};
+      foreach my $sortkey (keys %$entries) {
+        my $entry = $$entries{$sortkey};
+        $$split{ $$entry{initial} }{$sortkey} = $entry; }
+      @docs = map { $self->rescan($_) }
+        $self->makeSubCollectionDocuments($doc, $bib,
+        map { ($_ => $self->makeBibliographyList($doc, $bib, $_, $$split{$_})) }
+          sort keys %$split); }
+    else {
+      $doc->addNodes($bib, $self->makeBibliographyList($doc, $bib, undef, $entries));
+      #      @docs = ($self->rescan($doc)); }
+      $self->rescan($doc); } }
+  return @docs; }
 
 # Try to preserve the original form & case of the provided Bibkeys
 # HOWEVER, we downcase them before indexing & looking them up!!!!
@@ -95,15 +99,21 @@ sub normalizeBibKey {
 sub getBibliographies {
   my ($self, $doc) = @_;
   my @bibnames = ();
+  my $fromBibliography = 0; # coming from an 'ltx:bibliography'
   # use the commandline bibliographies, if explicitly given.
   if ($$self{bibliographies} && scalar(@{ $$self{bibliographies} })) {
     @bibnames = @{ $$self{bibliographies} }; }
   else {
     my $bibnode = $doc->findnode('//ltx:bibliography');
-    if (my $files = $bibnode->getAttribute('files')) {
-      @bibnames = map { $_ . '.bib' } split(',', $files); } }
-  my @paths = $doc->getSearchPaths;
-  my @bibs  = ();
+    if (my $files = $bibnode->getAttribute('files')
+      || $bibnode->parentNode->getAttribute('files')    # !!!!!
+    ) {
+      $fromBibliography = 1;
+      @bibnames = split(',', $files); } }
+  my @paths   = $doc->getSearchPaths;
+  my @bibs    = ();
+  my @rawbibs = ();
+ # Collect the "ready" bibliographies, while accumulating all raw sources for a single conversion pass
   foreach my $bib (@bibnames) {
     my $ref = ref $bib;
     my $bibdoc;
@@ -115,21 +125,23 @@ sub getBibliographies {
       Error('unexpected', $ref, $self,
         "Don't know how to convert '$bib' (a '$ref') into a Bibliography document"); }
     elsif (pathname_is_literaldata($bib)) {
-      # Assume (for now) that the data is BibTeX text;
-      # [another possibility is serialized xml!]
-      $bibdoc = $self->convertBibliography($doc, $bib); }
-    elsif ($bib =~ /\.xml/) {
-      $bibdoc = $doc->newFromFile($bib); }    # doc will do the searching...
-    elsif ($bib =~ /\.bib(?:\.xml)?$/) {
-      my $name = $1;
+      push(@rawbibs, $bib);
+      next; }
+    elsif ($bib =~ /\.xml$/) {
+      $bibdoc = $doc->newFromFile($bib); }                 # doc will do the searching...
+    elsif ($bib =~ /\.bib(?:\.xml)?$/ || $fromBibliography) {
       # NOTE: We should also use kpsewhich to get the effects of $BIBINPUTS?
       # NOTE: When better integrated with Core, should also check for cached bib documents.
-      if (my $xmlpath = pathname_find($bib, paths => [@paths], types => ['xml'])) {
+      my $xmlbib = $bib;
+      $xmlbib .= '.bib' if $fromBibliography && !($xmlbib =~ /\.bib$/);
+      if (my $xmlpath = pathname_find($xmlbib, paths => [@paths], types => ['xml'])) {
         $bibdoc = $doc->newFromFile($xmlpath); }    # doc will do the searching...
-      elsif (my $bibpath = pathname_find($bib, paths => [@paths], types => ['bib'])) {
-        $bibdoc = $self->convertBibliography($doc, $bibpath); }
+      elsif (my $bibpath = pathname_find($bib, paths => [@paths], types => ['bib'])
+        || pathname_kpsewhich($bib)) {
+        push(@rawbibs, $bibpath);
+        next; }
       else {
-        Error('expected', $bib, $self,
+        Error('missing_file', $bib, $self,
           "Couldn't find Bibliography '$bib'",
           "Searchpaths were " . join(',', @paths)); } }
     if ($bibdoc) {
@@ -137,7 +149,27 @@ sub getBibliographies {
     else {
       Info('expected', $bib, $self,
         "Couldn't find usable Bibliography for '$bib'"); } }
-  NoteProgress(" [using bibliographies "
+  # Lastly, If we found any raw .bib files/literaldata, convert and include them.
+  if (@rawbibs) {
+    my $raw;
+    if (scalar(@rawbibs) == 1) {    # Single, just convert as-is
+      $raw = $rawbibs[0]; }
+    else {
+      $raw = 'literal:';
+      for my $rawbib (@rawbibs) {    # Multiple, arrange into a single conversion payload
+        if ($rawbib =~ s/literal\://) {
+          $raw .= $rawbib; }
+        else {
+          # TODO: Is this a memory concern for large bib files?
+          if (open(my $bibfh, '<', $rawbib)) {
+            $raw .= join("", <$bibfh>);
+            close $bibfh; }
+          else {
+            Info("open", $rawbib, $self, "Couldn't open file $rawbib"); } }
+        $raw .= "%\n"; } }
+    my $bibdoc = $self->convertBibliography($doc, $raw);
+    push(@bibs, $bibdoc) if $bibdoc; }
+  NoteLog("MakeBibliography: using bibliographies "
       . join(',', map { (length($_) > 100 ? substr($_, 100) . '...' : $_) } @bibnames)
       . "]");
   return @bibs; }
@@ -155,31 +187,68 @@ sub convertBibliography {
   my ($self, $doc, $bib) = @_;
   require LaTeXML;
   require LaTeXML::Common::Config;
-  NoteProgress(" [Converting bibliography $bib ...");
+  my @preload = ();    # custom macros often used in e.g. howpublished field
+                       # need to preload all packages used by the main article
+  my ($classdata, @packages) = $self->find_documentclass_and_packages($doc);
+  my ($class, $classoptions) = @$classdata;
+  if ($class) {
+
+    if ($classoptions) {
+      push(@preload, "[$classoptions]$class.cls"); }
+    else {
+      push(@preload, "$class.cls"); } }
+  foreach my $po (@packages) {
+    my ($pkg, $options) = @$po;
+    if ($options) {
+      push(@preload, "[$options]$pkg.sty"); }
+    else {
+      push(@preload, "$pkg.sty"); } }
+  my $bibname = pathname_is_literaldata($bib) ? 'Anonymous Bib String' : $bib;
+  my $stage   = "Recursive MakeBibliography $bibname";
+  ProgressSpinup($stage);
   my $bib_config = LaTeXML::Common::Config->new(
+    recursive      => 1,
     cache_key      => 'BibTeX',
     type           => "BibTeX",
     post           => 0,
     format         => 'dom',
     whatsin        => 'document',
     whatsout       => 'document',
-    verbosity      => -5,
-    bibliographies => []);
+    includestyles  => 1,
+    bibliographies => [],
+    (@preload ? (preload => [@preload]) : ()));
   my $bib_converter = LaTeXML->get_converter($bib_config);
+  # Tricky and HACKY, we need to release the log to capture the inner workings separately.
+  # ->bind_log analog:
+  my $biblog = '';
+###  my $biblog_handle;
+###  open($biblog_handle, ">>", \$biblog) or Error("Can't redirect STDERR to log for inner bibliography converter!");
+###  *BIB_STDERR_SAVED = *STDERR;
+###  *STDERR           = *$biblog_handle;
+  # end ->bind_log
+
   $bib_converter->prepare_session($bib_config);
   my $response = $bib_converter->convert($bib);
-  print STDERR $$response{log};
-  # TODO: We need to handle the logging properly, it's a bit of a mess for nested ->convert() calls
+
+  # ->flush_log analog:
+###  close $biblog_handle;
+###  *STDERR = *BIB_STDERR_SAVED;
+  # end ->flush_log
+
+  # Trim log to look internal and report.
+###  $biblog =~ s/^.+?\(Digesting/\n\(Digesting/s;
+###  $biblog =~ s/Conversion complete:.+$//s;
+###  print STDERR $biblog;
+  MergeStatus($$bib_converter{latexml}{state});
+
+  ProgressSpindown($stage);
   if (my $bibdoc = $$response{result}) {
-    NoteProgress("... converted!]");
     return $doc->new($bibdoc, sourceDirectory => '.'); }
-  else {
-    NoteProgress("... Failed!]");
-    return; } }
+  return; }
 
 # ================================================================================
 # Get all cited bibentries from the requested bibliography files.
-# Sort (cited) bibentries on author+year+title, [NOT on the key!!!]
+# Sort (cited) bibentries on author+year+title, [NOT on the key, or even proper BibTeX choices!!!]
 # and then check whether author+year is unique!!!
 # Returns a list of hashes containing:
 #  bibkey : the bibliographic entry's key
@@ -187,38 +256,45 @@ sub convertBibliography {
 #  citations : array of bib keys that are cited somewhere within this bibentry
 #  referrers : array of ID's of places that refer to this bibentry
 #  suffix    : a,b... if adjacent author/year are identical.
-
+# NOTE: biblist's now have @lists to restrict to include ONLY
+# those bibitems which have been referenced by bibref's for one of those lists!
+# [If a bibref gives several lists, and entries for those lists end up in several bibliographies,
+# first 'list' determines where the idref points]
 sub getBibEntries {
-  my ($self, $doc) = @_;
+  my ($self, $doc, $bib) = @_;
 
   # First, scan the bib files for all ltx:bibentry's, (hash key is bibkey)
   # Also, record the citations from each bibentry to others.
   my %entries = ();
   foreach my $bibdoc ($self->getBibliographies($doc)) {
+    my @lists = split(/\s+/, $doc->findnode('//ltx:bibliography')->getAttribute('lists') || 'bibliography');
     foreach my $bibentry ($bibdoc->findnodes('//ltx:bibentry')) {
       my $bibkey = normalizeBibKey($bibentry->getAttribute('key'));
       my $bibid  = $bibentry->getAttribute('xml:id');
-      $entries{$bibkey}{bibkey}   = $bibkey;
-      $entries{$bibkey}{bibentry} = $bibentry;
+      $entries{$bibkey}{bibkey}    = $bibkey;
+      $entries{$bibkey}{bibentry}  = $bibentry;
       $entries{$bibkey}{citations} = [map { normalizeBibKey($_) } grep { $_ } map { split(',', $_->value) }
-          $bibdoc->findnodes('.//@bibrefs', $bibentry)];
-      # Also, register the key with the DB, if the bibliography hasn't already been scanned.
-      $$self{db}->register("BIBLABEL:$bibkey", id => $bibid);
-    } }
+          $bibdoc->findnodes('.//@bibrefs', $bibentry)]; } }
   # Now, collect all bibkeys that were cited in other documents (NOT the bibliography)
   # And note any referrers to them (also only those outside the bib)
-  my $citestar = $$self{db}->lookup('BIBLABEL:*');
+  my @lists    = split(/\s+/, $bib->getAttribute('lists') || 'bibliography');
+  my $citestar = grep { $$self{db}->lookup("BIBLABEL:$_:*"); } @lists;
 
   my @queue = ();
   foreach my $dbkey ($$self{db}->getKeys) {
-    if ($dbkey =~ /^BIBLABEL:(.*)$/) {
-      my $bibkey = $1;
-      if (my $referrers = $$self{db}->lookup($dbkey)->getValue('referrers')) {
+    if ($dbkey =~ /^BIBLABEL:(.*?):(.*)$/) {
+      my ($list, $bibkey) = ($1, $2);
+      next unless grep { $_ eq $list; } @lists;
+      my $bibdbentry = $$self{db}->lookup($dbkey);
+      if (my $referrers = $bibdbentry->getValue('referrers')) {
         foreach my $refr (keys %$referrers) {
           my ($rid, $e, $t) = ($refr, undef, undef);
           while ($rid && ($e = $$self{db}->lookup("ID:$rid")) && (($t = ($e->getValue('type') || '')) ne 'ltx:bibitem')) {
             $rid = $e->getValue('parent'); }
-          if ($t ne 'ltx:bibitem') {
+          if (!$e) {
+            Warn('expected', 'entry', undef,
+              "Didn't find an entry for reference id=$rid"); }
+          elsif ($t ne 'ltx:bibitem') {
             $entries{$bibkey}{referrers}{$refr} = 1; } }
         push(@queue, $bibkey) if keys %{ $entries{$bibkey}{referrers} }; }
       elsif ($citestar) {    # If \cite{*} include all of them.
@@ -245,16 +321,17 @@ sub getBibEntries {
       elsif (scalar(@names)) {
         $sortnames = join(' ', map { getNameText($doc, $_) } @names);
         my @ns = map { $_ && $_->textContent } map { $doc->findnodes('ltx:surname', $_) } @names;
-        if    (@ns > 2) { $names = $ns[0] . ' et al'; }
+        if (@ns > 2)    { $names = $ns[0] . ' et al'; }
         elsif (@ns > 1) { $names = $ns[0] . ' and ' . $ns[1]; }
-        else { $names = $ns[0]; } }
+        else            { $names = $ns[0]; } }
       elsif (my $t = $doc->findnode('ltx:bib-title', $bibentry)) {
         $sortnames = $names = $t->textContent; }
-      my $date = $doc->findnode('ltx:bib-date[@role="publication"] | ltx:bib-type', $bibentry);
-      my $title = $doc->findnode('ltx:bib-title', $bibentry);
-      $date  = ($date  ? $date->textContent  : '');
-      $title = ($title ? $title->textContent : '');
-      $$entry{ay} = "$names.$date";
+      my $date  = $doc->findnode('ltx:bib-date[@role="publication"] | ltx:bib-type', $bibentry);
+      my $title = $doc->findnode('ltx:bib-title',                                    $bibentry);
+      $date            = ($date ? $date->textContent : '');
+      $date            = $1 if $date && $date =~ /^(\d\d\d\d)/;
+      $title           = ($title ? $title->textContent : '');
+      $$entry{ay}      = "$names.$date";
       $$entry{initial} = $doc->initial($names, 1);
       # Include this entry keyed using a sortkey.
       $$included{ lc(join('.', $sortnames, $date, $title, $bibkey)) } = $entry;
@@ -268,14 +345,14 @@ sub getBibEntries {
     my $bibkey = $$entry{bibkey};
     map { $entries{ normalizeBibKey($_) }{bibreferrers}{$bibkey} = 1 } @{ $$entry{citations} }; }
 
-  NoteProgress(" [" . (scalar keys %entries) . " bibentries, " . (scalar keys %$included) . " cited]");
+  NoteLog("MakeBibliography: " . (scalar keys %entries) . " bibentries, " . (scalar keys %$included) . " cited");
   Warn('expected', 'bibkeys', undef,
     "Missing bibkeys " . join(', ', sort keys %missing_keys)) if keys %missing_keys;
 
   # Finally, sort the bibentries according to author+year+title+bibkey
   # If any neighboring entries have same author+year, set a suffix: a,b,...
   # Actually, it isn't so much if they are adjacent; if author+year isn't unique, need a suffix
-  my @sortkeys = sort keys %$included;
+  my @sortkeys = $doc->unisort(keys %$included);
   my %suffixes = ();
   while (my $sortkey = shift(@sortkeys)) {
     #    my $i=0;
@@ -310,22 +387,30 @@ sub getNameText {
 # Convert hash of bibentry(s) into biblist of bibitem(s)
 
 sub makeBibliographyList {
-  my ($self, $doc, $initial, $entries) = @_;
-  my $id = $doc->getDocumentElement->getAttribute('xml:id') || 'bib';
+  my ($self, $doc, $bib, $initial, $entries) = @_;
+  my $id = $bib->getAttribute('xml:id')
+    || $doc->getDocumentElement->getAttribute('xml:id') || 'bib';
   $id .= ".L1";
   $id .= ".$initial" if $initial;
   return ['ltx:biblist', { 'xml:id' => $id },
-    map { $self->formatBibEntry($doc, $$entries{$_}) } sort keys %$entries]; }
+    map { $self->formatBibEntry($doc, $bib, $$entries{$_}) } $doc->unisort(keys %$entries)]; }
 
 # ================================================================================
+# NOTE: With multiple bibliographies, the ID of the bibentry isn't necessarily
+# the ID in the "local" bibliography! (ie. bibentry can be repeated in the document!)
 sub formatBibEntry {
-  my ($self, $doc, $entry) = @_;
+  my ($self, $doc, $bib, $entry) = @_;
   my $bibentry   = $$entry{bibentry};
   my $id         = $bibentry->getAttribute('xml:id');
   my $key        = $bibentry->getAttribute('key');
   my $type       = $bibentry->getAttribute('type');
   my @blockspecs = @{ $FMT_SPEC{$type} || [] };
 
+  # Patch the entry's id in case there are multiple bibs.
+  if (my $bibid = $bib->getAttribute('xml:id')
+    || $doc->getDocumentElement->getAttribute('xml:id') || 'bib') {
+    $id =~ s/^bib//; $id = $bibid . $id; }
+
   local $LaTeXML::Post::MakeBibliography::SUFFIX = $$entry{suffix};
   my $number = ++$LaTeXML::Post::MakeBibliography::NUMBER;
 
@@ -335,33 +420,36 @@ sub formatBibEntry {
   #------------------------------
   # Format the bibtag's
   my @tags = ();
-  push(@tags, ['ltx:bibtag', { role => 'number', class => 'ltx_bib_number' }, $number]);  # number tag
+  push(@tags, ['ltx:tag', { role => 'number', class => 'ltx_bib_number' }, $number]);    # number tag
 
   # Set up authors and fullauthors tags
   my @names = $doc->findnodes('ltx:bib-name[@role="author"]/ltx:surname', $bibentry);
   @names = $doc->findnodes('ltx:bib-name[@role="editor"]/ltx:surname', $bibentry) unless @names;
+  my $etal = 0;
+  if (@names && ($names[-1]->toString eq 'others')) {                                    # Magic!
+    $etal = 1; }
   if (@names > 2) {
-    push(@tags, ['ltx:bibtag', { role => 'authors', class => 'ltx_bib_author' },
+    push(@tags, ['ltx:tag', { role => 'authors', class => 'ltx_bib_author' },
         $doc->cloneNodes($names[0]->childNodes),
         ['ltx:text', { class => 'ltx_bib_etal' }, ' et al.']]);
     my @fnames = ();
     foreach my $n (@names[0 .. $#names - 1]) {
       push(@fnames, $n->childNodes, ', '); }
-    push(@tags, ['ltx:bibtag', { role => 'fullauthors', class => 'ltx_bib_author' },
+    push(@tags, ['ltx:tag', { role => 'fullauthors', class => 'ltx_bib_author' },
         $doc->cloneNodes(@fnames),
         ' and ', $doc->cloneNodes($names[-1]->childNodes)]); }
   elsif (@names > 1) {
-    push(@tags, ['ltx:bibtag', { role => 'authors', class => 'ltx_bib_author' },
+    push(@tags, ['ltx:tag', { role => 'authors', class => 'ltx_bib_author' },
         $doc->cloneNodes($names[0]->childNodes),
         ' and ', $doc->cloneNodes($names[1]->childNodes)]); }
   elsif (@names) {
-    push(@tags, ['ltx:bibtag', { role => 'authors', class => 'ltx_bib_author' },
+    push(@tags, ['ltx:tag', { role => 'authors', class => 'ltx_bib_author' },
         $doc->cloneNodes($names[0]->childNodes)]); }
 
   # Put a key tag, to use in place of authors if needed (esp for software, websites, etc)
   my $keytag;
   if ($keytag = $doc->findnode('ltx:bib-key', $bibentry)) {
-    push(@tags, ['ltx:bibtag', { role => 'key', class => 'ltx_bib_key' },
+    push(@tags, ['ltx:tag', { role => 'key', class => 'ltx_bib_key' },
         $doc->cloneNodes($keytag->childNodes)]); }
 
   my @year = ();
@@ -370,18 +458,18 @@ sub formatBibEntry {
     if (my $datetext = $date->textContent) {
       if ($datetext =~ /^(\d\d\d\d)/) {    # Extract 4 digit year, if any
         @year = ($1); } }
-    push(@tags, ['ltx:bibtag', { role => 'year', class => 'ltx_bib_year' },
+    push(@tags, ['ltx:tag', { role => 'year', class => 'ltx_bib_year' },
         $doc->cloneNodes(@year), ($$entry{suffix} || '')]); }
 
   # Store a type tag, to use in place of year, if needed (esp for software, ...)
   my $typetag;
   if ($typetag = $doc->findnode('ltx:bib-type', $bibentry)) {
-    push(@tags, ['ltx:bibtag', { role => 'bibtype', class => 'ltx_bib_type' },
+    push(@tags, ['ltx:tag', { role => 'bibtype', class => 'ltx_bib_type' },
         $doc->cloneNodes($typetag->childNodes)]); }
 
   # put in the title
   if (my $title = $doc->findnode('ltx:bib-title', $bibentry)) {
-    push(@tags, ['ltx:bibtag', { role => 'title', class => 'ltx_bib_title' },
+    push(@tags, ['ltx:tag', { role => 'title', class => 'ltx_bib_title' },
         $doc->cloneNodes($title->childNodes)]); }
 
   # And finally, the refnum; we need to know the desired citation style!
@@ -390,23 +478,25 @@ sub formatBibEntry {
   my $style = $LaTeXML::Post::MakeBibliography::STYLE{citestyle} || 'numbers';
   $style = 'numbers' unless (@names || $keytag) && (@year || $typetag);
   if ($style eq 'numbers') {
-    push(@tags, ['ltx:bibtag', { role => 'refnum', class => 'ltx_bib_key', open => '[', close => ']' }, $number]); }
+    push(@tags, ['ltx:tag', { role => 'refnum', class => 'ltx_bib_key', open => '[', close => ']' }, $number]); }
   elsif ($style eq 'AY') {
     my @rfnames;
-    if (my @authors = $doc->findnodes('ltx:bib-name[@role="author"]', $bibentry)) {
+    if (my @authors = $doc->findnodes('ltx:bib-name[@role="author"]/ltx:surname', $bibentry)) {
       @rfnames = @authors; }
-    elsif (my @editors = $doc->findnodes('ltx:bib-name[@role="editor"]', $bibentry)) {
+    elsif (my @editors = $doc->findnodes('ltx:bib-name[@role="editor"]/ltx:surname', $bibentry)) {
       @rfnames = @editors; }
     else {
       @rfnames = $keytag->childNodes; }
     my $aa;
     if (scalar(@rfnames) > 1) {
-      $aa = join('', map { substr($_->textContent, 0, 1); } @rfnames); }
+      $aa = join('', map { substr($_->textContent, 0, 1); } @rfnames);
+      if (length($aa) > 3) {
+        $aa = substr($aa, 0, 3) . "+"; } }
     else {
       $aa = uc(substr($rfnames[0]->textContent, 0, 3)); }
     my $yrtext = (@year ? join('', map { (ref $_ ? $_->textContent : $_); } @year) : '');
-    my $yy = (length($yrtext) >= 2 ? substr($yrtext, 2, 2) : $yrtext);
-    push(@tags, ['ltx:bibtag', { role => 'refnum', class => 'ltx_bib_abbrv', open => '[', close => ']' },
+    my $yy     = (length($yrtext) >= 2 ? substr($yrtext, 2, 2) : $yrtext);
+    push(@tags, ['ltx:tag', { role => 'refnum', class => 'ltx_bib_abbrv', open => '[', close => ']' },
         $aa . $yy . ($$entry{suffix} || '')]); }
 
   else {
@@ -420,7 +510,7 @@ sub formatBibEntry {
       @rfnames = $keytag->childNodes; }
     my @rfyear = (@year ? (@year, ($$entry{suffix} || ''))
       : ($typetag ? $typetag->childNodes : ()));
-    push(@tags, ['ltx:bibtag', { role => 'refnum', class => 'ltx_bib_author-year' },
+    push(@tags, ['ltx:tag', { role => 'refnum', class => 'ltx_bib_author-year' },
         $doc->cloneNodes(@rfnames), ' (', $doc->cloneNodes(@rfyear), ')']); }
 
   #------------------------------
@@ -431,17 +521,17 @@ sub formatBibEntry {
     foreach my $row (@$blockspec) {
       my ($xpath, $punct, $pre, $class, $op, $post) = @$row;
       my $negated = $xpath =~ s/^!\s*//;
-      my @nodes = ($xpath eq 'true' ? () : $doc->findnodes($xpath, $bibentry));
+      my @nodes   = ($xpath eq 'true' ? () : $doc->findnodes($xpath, $bibentry));
       next if ($xpath ne 'true') && ($negated ? @nodes : !@nodes);
       push(@x, $punct) if $punct && @x;
-      push(@x, $pre) if $pre;
+      push(@x, $pre)                                                                           if $pre;
       push(@x, ['ltx:text', { class => 'ltx_bib_' . $class }, &$op($doc->cloneNodes(@nodes))]) if $op;
-      push(@x, $post) if $post; }
+      push(@x, $post)                                                                          if $post; }
     push(@blocks, ['ltx:bibblock', { 'xml:space' => 'preserve' }, @x]) if @x;
   }
   # Add a Cited by block.
 
-  my @citedby = map { ['ltx:ref', { idref => $_, show => 'rrefnum' }] }
+  my @citedby = map { ['ltx:ref', { idref => $_, show => 'typerefnum' }] }
     sort keys %{ $$entry{referrers} };
   push(@citedby, ['ltx:bibref', { bibrefs => join(',', sort keys %{ $$entry{bibreferrers} }), show => 'refnum' }])
     if $$entry{bibreferrers};
@@ -449,7 +539,7 @@ sub formatBibEntry {
       "Cited by: ", $doc->conjoin(",\n", @citedby), '.']) if @citedby;
 
   return ['ltx:bibitem', { 'xml:id' => $id, key => $key, type => $type, class => "ltx_bib_$type" },
-    @tags,
+    (@tags ? (['ltx:tags', {}, @tags]) : ()),
     @blocks]; }
 
 # ================================================================================
@@ -474,10 +564,20 @@ sub do_name {
 
 sub do_names {
   my (@names) = @_;
-  my @stuff = ();
+  my @stuff   = ();
+  my $sep     = (scalar(@names) > 2 ? ', ' : ' ');
+  my $etal    = 0;
+  if (@names && ($names[-1]->textContent eq 'others')) {    # Magic!
+    pop(@names);
+    $etal = 1; }
+  my $n = scalar(@names);
   while (my $name = shift(@names)) {
-    push(@stuff, (@names ? ', ' : ' and ')) if @stuff;
+    if (@stuff) {
+      push(@stuff, $sep);
+      push(@stuff, 'and ') if !$etal && !@names; }
     push(@stuff, do_name($name)); }
+  if ($etal) {
+    push(@stuff, $sep, ['ltx:text', { class => 'ltx_bib_etal' }, 'et al.']); }
   return @stuff; }
 
 sub do_names_short {
@@ -509,7 +609,7 @@ sub do_editorsB {
 
 sub do_year {
   my (@stuff) = @_;
-  return ('(', @stuff, @LaTeXML::Post::MakeBibliography::SUFFIX, ')'); }
+  return (' (', @stuff, @LaTeXML::Post::MakeBibliography::SUFFIX, ')'); }
 
 sub do_type {
   my (@stuff) = @_;
@@ -536,7 +636,7 @@ sub do_pages {
 sub do_crossref {
   my ($node, @stuff) = @_;
   return (['ltx:cite', {},
-      ['ltx:bibref', { bibrefs => $node->getAttribute('bibrefs'), show => 'refnum' }]]); }
+      ['ltx:bibref', { bibrefs => $node->getAttribute('bibrefs'), show => 'title, author' }]]); }
 
 my $LINKS =                                       # CONSTANT
   "ltx:bib-links | ltx:bib-review | ltx:bib-identifier | ltx:bib-url";
@@ -626,10 +726,10 @@ sub do_links {
       ['ltx:bib-related[@bibrefs]', ' ', 'See ', 'crossref', \&do_crossref, ','],
       # if NO crossref, used embedded editors & booktitle.
       #      ['ltx:bib-related[@type="book"]/ltx:bib-title', ' ', 'in ', 'inbook',   \&do_title,    ',']
-      ['ltx:bib-related[@type="book"][not(../ltx:bib-related[@bibrefs])]/ltx:bib-name[@role="editor"]',
-        ' ', 'in ', 'editor', \&do_editorsA, ','],
-      ['ltx:bib-related[@type="book"][not(../ltx:bib-related[@bibrefs])]/ltx:bib-title',
-        ' ', '', 'inbook', \&do_title, ',']
+      ['ltx:bib-related[@type][not(../ltx:bib-related[@bibrefs])]/ltx:bib-title',
+        ' ', 'In ', 'inbook', \&do_title, ','],
+      ['ltx:bib-related[@type][not(../ltx:bib-related[@bibrefs])]/ltx:bib-name[@role="editor"]',
+        ' ', ' ', 'editor', \&do_editorsA, ','],
     ],
     [['ltx:bib-edition', '', '', 'edition', \&do_edition, ''],
       ['ltx:bib-name[@role="editor"]',                 ', ', '',      'editor',    \&do_editorsB, ''],
@@ -681,7 +781,7 @@ sub do_links {
       ['ltx:bib-date[@role="publication"]', '', '', 'year',   \&do_year,     ''],
       ['ltx:title',                         '', '', 'title',  \&do_any,      ''],
       ['ltx:bib-type',                      '', '', 'type',   \&do_any,      ''],
-      ['! ltx:bib-type', '', '', 'type', sub { ('(Website)'); }, '']],
+      ['! ltx:bib-type',                    '', '', 'type',   sub { ('(Website)'); }, '']],
     [['ltx:bib-organization', ', ', ' ', 'publisher', \&do_any, ''],
       ['ltx:bib-place', ', ', '', 'place', \&do_any, ''],
       ['true', '.']],
@@ -713,4 +813,3 @@ $FMT_SPEC{techreport}            = $FMT_SPEC{report};
 
 # ================================================================================
 1;
-
