@@ -58,7 +58,7 @@ sub copy_attributes_except {
   my %excluded = map { ($_ => 1) } @attributes;
   foreach my $attr ($from->attributes) {
     my $key = $attr->getName;
-    next if $excluded{$key};
+    next                         if $excluded{$key};
     $from->removeAttribute($key) if $key eq 'xml:id';
     $to->setAttribute($key, $from->getAttribute($key)); }
   return; }
@@ -95,13 +95,16 @@ sub makeViewBox {
   if ($w =~ /^($NR)([a-z]{2})$/) { $w = $1; }
   if ($h =~ /^($NR)([a-z]{2})$/) { $h = $1; }
   my ($minx, $maxx, $miny, $maxy) = map { $_ || 0 } @{ getSVGBounds($node) };
+  $minx = $node->getAttribute('origin-x') || 0; $minx =~ s/pt$//;
+  $miny = $node->getAttribute('origin-y') || 0; $miny =~ s/pt$//;
   my $ww = $maxx - $minx;
   my $hh = $maxy - $miny;
-###  $node->setAttribute(viewBox=>"$minx $miny $w $h");
-
-  $node->setAttribute(width  => $ww) if $ww > $w;
-  $node->setAttribute(height => $hh) if $hh > $h;
-  $node->setAttribute(viewBox => "$minx $miny $maxx $maxy");
+  $node->setAttribute(width  => $ww . "pt") if $ww > $w;
+  $node->setAttribute(height => $hh . "pt") if $hh > $h;
+  $maxx = $w if $ww < 1;
+  $maxy = $h if $hh < 1;
+  # use 0,0 for origin, not minx,miny
+  $node->setAttribute(viewBox => "0 0 $maxx $maxy");
 
   $node->setAttribute(overflow => 'visible') if (($node->getAttribute('clip') || '') ne 'true');
   $node->removeAttribute('clip');
@@ -133,13 +136,13 @@ sub simplifyGroups {
 # Converting specific tags.
 #======================================================================
 my %converters = (    # CONSTANT
-  'ltx:picture'      => \&convertPicture, 'ltx:path'   => \&convertPath,
-  'ltx:g'            => \&convertG,       'ltx:text'   => \&convertText,
-  'ltx:polygon'      => \&convertPolygon, 'ltx:line'   => \&convertLine,
-  'ltx:rect'         => \&convertRect,    'ltx:bezier' => \&convertBezier,
-  'ltx:inline-block' => \&convertVbox,    'ltx:circle' => \&convertCircle,
-  'ltx:ellipse'      => \&convertEllipse, 'ltx:wedge'  => \&convertWedge,
-  'ltx:arc'          => \&convertArc,     'ltx:dots'   => \&convertDots);
+  'ltx:picture' => \&convertPicture, 'ltx:path'   => \&convertPath,
+  'ltx:g'       => \&convertG,       'ltx:text'   => \&convertText,
+  'ltx:polygon' => \&convertPolygon, 'ltx:line'   => \&convertLine,
+  'ltx:rect'    => \&convertRect,    'ltx:bezier' => \&convertBezier,
+  'ltx:circle'  => \&convertCircle,
+  'ltx:ellipse' => \&convertEllipse, 'ltx:wedge' => \&convertWedge,
+  'ltx:arc'     => \&convertArc,     'ltx:dots'  => \&convertDots);
 
 sub convertNode {
   my ($parent, $node) = @_;
@@ -149,14 +152,26 @@ sub convertNode {
   elsif (my $converter = $converters{$tag}) {
     &$converter($parent, $node); }
   else {
-    # my $new = $parent->addNewChild($svgURI,'foreignObject');
-    # $new->appendChild($node); }}
+    # Node is random LaTeXML element, and so will need an svg:foreignObject wrapper.
+    # Moreover, svg will want to know the size of the foreign thing.
+    # Hopefully, this will have been recorded on a containing ltx:g, using innerwidth/innerheight.
     my $g = $parent->addNewChild($svgURI, 'g');
-    $g->setAttribute(transform => "scale(1 -1) translate(-5,-10)");    # AD HOC
-    my $new = $g->addNewChild($svgURI, 'foreignObject');
-    # Totally wrong, but until we properly size things, we HAVE to give it SOME size!
-    $new->setAttribute(width  => 50);
-    $new->setAttribute(height => 20);
+    $g->setAttribute(transform => "scale(1 -1)");
+    my $new       = $g->addNewChild($svgURI, 'foreignObject');
+    my $oldparent = $node->parentNode;
+    my $width     = $node->getAttribute('width')                 # Use node's own width, if any
+      || $node->getAttribute('imagewidth')                       # or if an image
+      || $oldparent->getAttribute('innerwidth')                  # else hopefully from containing ltx:g
+      || $oldparent->getAttribute('width');
+    my $height = $node->getAttribute('height')
+      || $node->getAttribute('imageheight')
+      || $oldparent->getAttribute('innerheight')
+      || $oldparent->getAttribute('height');
+    $width  = "50pt" unless defined $width;
+    $height = "20pt" unless defined $height;
+    $new->setAttribute(width    => $width);
+    $new->setAttribute(height   => $height);
+    $new->setAttribute(overflow => 'visible');
     $new->appendChild($node); }
   return; }
 
@@ -182,7 +197,7 @@ sub convertPicture {
   #  my ($minx, $maxx, $miny, $maxy) = map { $_ || 0 } @{getSVGBounds($node)};
   #  my $h = $maxy-$miny;
 
-  my $h = $node->getAttribute('height') || '0'; $h =~ s/pt$//;
+  my $h      = $node->getAttribute('height') || '0'; $h =~ s/pt$//;
   my $mvNode = $parent->addNewChild($svgURI, 'g');
   $mvNode->setAttribute(transform => "translate(0,$h)");
 
@@ -213,7 +228,7 @@ sub convertText {
   my $p = ((getQName($oldparent) || '') eq 'ltx:g' ? $oldparent->getAttribute('pos') || '' : 'bl');
   my $newNode = $parent->addNewChild($svgURI, 'text');
   $newNode->setAttribute('dominant-baseline' => 'middle');
-  $newNode->setAttribute('baseline-shift'    => 'sub') if $p =~ /t/;
+  $newNode->setAttribute('baseline-shift'    => 'sub')   if $p =~ /t/;
   $newNode->setAttribute('baseline-shift'    => 'super') if $p =~ /b/;
   if ($p =~ /l/) {
     $newNode->setAttribute('text-anchor' => 'start'); }
@@ -298,9 +313,9 @@ sub convertRect {
 
 sub convertBezier {
   my ($parent, $node) = @_;
-  my @p = explodeCoord($node->getAttribute('points') || '');
-  my $n = ($#p + 1) / 2; my $x0 = shift(@p); my $y0 = shift(@p);
-  my %cmd = (4 => 'C', 3 => 'Q');
+  my @p       = explodeCoord($node->getAttribute('points') || '');
+  my $n       = ($#p + 1) / 2; my $x0 = shift(@p); my $y0 = shift(@p);
+  my %cmd     = (4 => 'C', 3 => 'Q');
   my $newNode = $parent->addNewChild($svgURI, 'path');
   $newNode->setAttribute(d => "M $x0,$y0 " . ($cmd{$n} || 'T') . ' ' . coordList(@p));
   copy_some_attributes($newNode, $node,
@@ -337,7 +352,7 @@ sub convertCircle {
 sub convertDots {
   my ($parent, $node) = @_;
   my $newNode = $parent->addNewChild($svgURI, 'g');
-  my @p = explodeCoord($node->getAttribute('points') || '');
+  my @p       = explodeCoord($node->getAttribute('points') || '');
   while (@p) {
     my ($x, $y) = (shift(@p), shift(@p));
     my $dot = $newNode->addNewChild($svgURI, 'circle');
@@ -441,7 +456,7 @@ sub setArrows {
 sub mergeTransform {
   my ($node, $new_t) = @_;
   my $old_t = $node->getAttribute('transform');
-  my $t = ($old_t && $new_t) ? Transform("$new_t $old_t")->toString :
+  my $t     = ($old_t && $new_t) ? Transform("$new_t $old_t")->toString :
     ($old_t ? $old_t : ($new_t ? $new_t : undef));
   $node->setAttribute(transform => $t) if $t;
   return; }
@@ -483,6 +498,8 @@ sub boxContentPos {
   my ($node) = @_;
   my ($nw, $nh, $npos) = get_attr($node, qw(width height pos));
   return (0, 0) unless defined $nw && defined $nh;
+  $nw =~ s/pt$//;
+  $nh =~ s/pt$//;
   if (!$npos) {
     return ($nw / 2, $nh / 2); }
   elsif ($npos eq 't') {
@@ -508,14 +525,16 @@ sub arcPoints {
   return 'M ' . $pts if !$r && $pts;
   local *getP = sub {
     my ($x1, $y1, $x2, $y2) = @_;
-    my $dst = sqrt(($x1 - $x2)**2 + ($y1 - $y2)**2);
-    my $s = ($x2 - $x1) * ($y2 - $y1) >= 0 ? 1 : -1;
+    # TODO: Do we need a warning if $dst is zero?
+    #       Default to 0.01 since we'll use it as a denominator
+    my $dst = sqrt(($x1 - $x2)**2 + ($y1 - $y2)**2) || 0.01;
+    my $s   = ($x2 - $x1) * ($y2 - $y1) >= 0 ? 1 : -1;
     trunc(2, $s, $x1 + ($x2 - $x1) * $r / $dst, $y1 + ($y2 - $y1) * $r / $dst); };
   my @p = explodeCoord($pts); my $n = ($#p + 1) / 2;
   my $d = "M $p[0] $p[1] ";
   for (my $i = 1 ; $i < $n - 1 ; $i++) {
-    my ($x2, $y2) = ($p[2 * $i - 2], $p[2 * $i - 1]);
-    my ($x1, $y1) = ($p[2 * $i], $p[2 * $i + 1]);
+    my ($x2, $y2)      = ($p[2 * $i - 2], $p[2 * $i - 1]);
+    my ($x1, $y1)      = ($p[2 * $i], $p[2 * $i + 1]);
     my ($sa, $xa, $ya) = getP($x1, $y1, $x2, $y2);
     ($x2, $y2) = ($p[2 * $i + 2], $p[2 * $i + 3]);
     my ($sb, $xb, $yb) = getP($x1, $y1, $x2, $y2);
@@ -569,7 +588,11 @@ sub SVGObjectBoundary {
       elsif (($mode eq 'xy' && $data =~ s/^\s*($NR)\s+($NR)\s*//) ||
         ($mode eq 'i5xy' && $data =~ s/^\s*$NR\s+$NR\s+$NR
          \s+$NR\s+$NR\s+($NR)\s+($NR)\s*//x)) {
-        push(@xs, $1); push(@ys, $2); } } }
+        push(@xs, $1); push(@ys, $2); }
+      else {
+        Error('unexpected', 'path', undef,
+          "Unrecognized svg path in '" . $node->getAttribute('d') . "' at '$data'");
+        last; } } }
   elsif ($tag eq 'svg:rect') {
     my ($x, $y, $w, $h) = get_attr($node, qw(x y width height));
     if (defined $x && defined $y && defined $w && defined $h) {
@@ -580,6 +603,8 @@ sub SVGObjectBoundary {
     push(@xs, $ex - $rx, $ex + $rx); push(@ys, $ey - $ry, $ey + $ry); }
   elsif ($tag eq 'svg:foreignObject') {
     my ($w, $h) = get_attr($node, qw (width height));
+    $w =~ s/pt$//;
+    $h =~ s/pt$//;
     push(@xs, 0, $w); push(@ys, 0, $h); }
 
   @xs = grep { defined $_ } @xs;
@@ -595,7 +620,7 @@ sub SVGObjectBoundary {
 sub combBoundary {
   my ($aa, $bb) = @_;
   return unless @$bb;
-  @$aa = @$bb and return unless @$aa;
+  @$aa    = @$bb and return unless @$aa;
   $$aa[0] = $$bb[0] if (!defined $$aa[0] || (defined $$bb[0] && $$aa[0] > $$bb[0]));
   $$aa[2] = $$bb[2] if (!defined $$aa[2] || (defined $$bb[2] && $$aa[2] > $$bb[2]));
   $$aa[1] = $$bb[1] if (!defined $$aa[1] || (defined $$bb[1] && $$aa[1] < $$bb[1]));
